# checks presence of one ot two "terminal" edges in thin wire
# output to a figure.

from pieces import W_upper, W_lower, W_point_x, W_point_y, W_point_z
from basic_routines import map_coordinates
import arithmetic
from arithmetic import print_scaled_int, decimal_to_scaled_int
 
import mwt_polygon

from output_routines import (print_triangles, print_polygon, print_points,
     draw_piece, startipefile, finishipefile)

def drawoutline(outfilename="results/set-W-outline.ipe"):   
   out=startipefile(outfilename)
   out.write('<group matrix="7 0 0 7 200 280">')
   out.write(print_polygon(W_lower+W_upper, closed=True,
     options = 'stroke="black" fill="gray7" dash="void" pen="normal"'))
   out.write(print_polygon(W_upper,closed=False))
   out.write(print_polygon(W_lower,closed=False))
   out.write('<group>')
   for p in W_lower+W_upper:
       out.write('<text stroke="black" pos="%f %f" type="label" valign="bottom"'
  ' transformable="no" size="small">$(%s,%s)$</text>\n' %
                 (p[0],p[1],str(p[0]-W_lower[7][0]),str(p[1])) )
   out.write('</group>')
   out.write('<group>')
   for i in range(7):
       out.write('<text stroke="black" pos="%f %f" type="label" valign="bottom"'
  ' transformable="no" valign="top" size="normal">$v_%d$</text>\n' %
                 (W_lower[i][0],W_lower[i][1],i) )
       out.write('<text stroke="black" pos="%f %f" type="label" valign="bottom"'
  ''' transformable="no" valign="top" size="normal">$v'_%d$</text>\n''' %
                 (W_lower[-i-1][0],W_lower[-i-1][1],i) )
   out.write('</group>')
   out.write(print_points(W_lower+W_upper))
   out.write('</group>')
   finishipefile(out)

  
def check_W_cases(outfilename="results/wire-cases.ipe"):
   mindiff = 999 * arithmetic.scale
   
   out=startipefile(outfilename)
   number = 0
   row = 15
   column = 0
   

   ijsequence = [(i,i) for i in range(1,7)]+ \
                [(i,j) for i in range(1,7) for j in range(i+1,7)]
   for i,j in ijsequence: #  for 1<=i<=j<=6:
       p = W_lower[i:len(W_lower)-j] + W_upper

       pcalc = map_coordinates(decimal_to_scaled_int, p)

       i_x = p.index(W_point_x);
       i_y = p.index(W_point_y);
       i_z = p.index(W_point_z)
       excl = [(i_x,i_y),(i_x,i_z)] # critical edges

       # First, compute optimum solution without restriction
       elist = mwt_polygon.mwt(pcalc)

       for (result,sol,degenerate) in elist:
          # convert list of triplets to list
          # of pairs representing edges:
          sole = [(x,y) for [x,y,z] in sol] + \
                 [(y,z) for [x,y,z] in sol]
          if not((i_x,i_y) in sole or (i_x,i_z) in sole or
                 (i_y,i_x) in sole or (i_z,i_x) in sole):
              print "incorrect:",i,j,result,sol,degenerate
              print ("optimal solution does not contain" ,
                        i_x,i_y, "or", i_x,i_z)
              raise StandardError
          if degenerate:
              print "DEGENERATE SOLUTION:",i,j,result,sol,degenerate
              raise StandardError

       # Now, compute optimum solution with excluded edges
       elist2 = mwt_polygon.mwt(pcalc, excluded=excl)

       diff = elist2[0][0].a - max(result.b for (result,sol,f) in elist)
       mindiff = min(mindiff,diff)
       print i,j,print_scaled_int(diff)

       out.write('<group layer="alpha" matrix="1.38 0 0 1.38 %f %5.3f">\n' %
                  (80+250*column, 37*row))
       row -=1
       if row==0:
           row=6
           column +=1
       number += 1
       out.write(
         ('<path stroke="black" pen="heavier">-12 -13 m -12 13.81 l</path>\n'
          '<path stroke="black" pen="normal">-40 -13 m -40 13.81 l</path>\n'
          '<text stroke="black" pos="-15 9" type="label" valign="top"'
          ' halign="right" transformable="no" size="normal">'
          r"\vbox{\openup 2\jot\halign{\hfil #\cr$v_%d,v'_%d$\hfil\cr \small %s\cr}}"
          '</text>\n')%
                (i,j,print_scaled_int(diff)))
       count = 0
       for (erg,sol,f) in elist2[:1]+elist:
           # only the first non-optimal is output
         val = (r"\vbox{\small\halign{\hfil$#$&amp;#\hfil\cr [%s&amp;,\cr %s&amp;]\cr}}"
                 % (print_scaled_int(erg.a), print_scaled_int(erg.b)))
         count += 1
         if f: val += '**'
         out.write('<group layer="alpha" matrix="1 0 0 1 %5.3f 0">\n'
                   % (71*(count-1)))
         out.write(print_triangles(p,sol, val,pos=(27,-2) ))
         if count==1: out.write('<path stroke="black" pen="heavier">')
         else:        out.write('<path stroke="black" pen="normal">')
         out.write('60 -13 m 60 13.81 l</path>\n')
         out.write('</group>\n')
       
       out.write('<path stroke="black" pen="normal">-40 -13 m %f -13 l</path>\n'
                 '<path stroke="black" pen="normal">-40 13.81 m %f 13.81 l</path>\n'
                 % (2*(-11+71*count,)))
       out.write('</group>\n')
   out.write('<path stroke="black" pen="normal">'
              '24.8 19.06 m 510.78 19.06 l 510.78 574.058 l 24.8 574.058 l h'
              '</path>\n')
   finishipefile(out)
   print "min. difference: ", print_scaled_int(mindiff)
drawoutline()
arithmetic.setprecision(5)
check_W_cases()
